function [z1,z2]=top_base(x,pts,N)
% function [z1,z2]=top_base_IEEE(x,pts,N)
% finds the top line and base line of a signal
%             pts - number of points per interval - needs to be an integer
%             N - number of intervals with consecutive 1s or 0s
% extracts N consecutive ones, and N consecutive zeros
% finds the histogram of the two, and the most frequent values for both
% The most frequent values of the histograms are the top line and the base line (z1 and z2)
% the difference of the two is the OMA - calculated outside of the program
%
% dependencies: needs the function zero_crossings 
% 
% Author: Petar Pepeljugoski
% 


% default value for the N consecutive zeros (or ones)
if nargin<3
    N=5;
end

% the number of samples per bit is required
if nargin<2
    disp('You must provide the number of samples per interval, exiting')
    return
end

if rem(pts, floor(pts))~=0
    disp('The number of samples per bit must be an integer - exiting the program')
    return
end


x=x(:);
x=x'; % make sure the data is in row format

x=[x, x, x, x]; % make sure there is sufficient number of zero crossings

% find the zero crossings
zcr=zero_crossings(x,pts);


% find the signal that has more than N ones or zeros
[j,i]=find(diff(zcr)>(N*pts-N*pts/4));
for k=2:length(zcr);
    int(k-1)=zcr(k)-zcr(k-1);
end


% extract the first N consecutive symbols (either zeros or ones)
tmp=x(zcr(i(1)):zcr(i(2))); 

if mean(tmp)>0
    newxp=tmp;
    newxn=[];
elseif mean(tmp)<0
    newxp=[];
    newxn=tmp;
end;

% continue with the rest, separating the ones and zeros
for k=2:(length(i)), 
    tmp=x(zcr(i(k)):zcr(i(k)+1));
    if mean(tmp)>0
        newxp=[newxp tmp];
    elseif mean(tmp)<0
        newxn=[newxn tmp];
    end
    tmp=[];    
end;

% select the number of bins in the histogram
if length(newxp)>1000
    Nhistp=100;
else
    Nhistp=floor(length(newxp)/10);
end
if length(newxn)>1000
    Nhistn=100;
else
    Nhistn=floor(length(newxn)/10);
end


% find the histograms
[Mp,Np]=hist(newxp,Nhistp);
[Mn,Nn]=hist(newxn,Nhistn);

% and their signal most frequent samples
[Mp1,Np1]=max(Mp);
[Mn1,Nn1]=max(Mn);

% and the top line and base line
z1=Np(Np1);
z2=Nn(Nn1);